package edu.cornell.cs.cs4120.xth.xic.tester;

import java.io.File;
import java.util.List;

import edu.cornell.cs.cs4120.xth.PDFReporter;
import edu.cornell.cs.cs4120.xth.SourceFileTest;

public class XicExtensionTester extends XicCodeGenTester {

    protected boolean isInvalidProgram(SourceFileTest t) {
        return !t.getExpectedFailures().isEmpty();
    }

    @Override
    public boolean copyReferenceFiles(SourceFileTest t, File testDir,
            File destDir) {
        boolean okay = true;
        if (isInvalidProgram(t))
            // Nothing to copy
            return okay;
        String testDirname = appendDirSep(testDir.getPath());
        String destDirname = appendDirSep(destDir.getPath());
        for (List<String> compilationUnit : t.getSourceFileNames())
            for (String filename : compilationUnit) {
                String ssolFilename =
                        normalizedFilename(ssolFilename(filename));
                okay = okay && copyFile(testDirname + ssolFilename,
                                        destDirname + ssolFilename,
                                        t);
            }
        return okay;
    }

    @Override
    public boolean renameReferenceFiles(SourceFileTest t, File destDir) {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean normalizeGeneratedFiles(SourceFileTest t, File destDir) {
        if (isInvalidProgram(t))
            // Nothing to normalize
            return true;
        File exeFile = prepareExecutable(t, destDir);
        if (exeFile == null) return false;
        String destDirname = appendDirSep(destDir.getPath());
        String sFilename = sFilename(exeFile.getName());
        String snmlFilename = normalizedFilename(sFilename);
        File snmlFile = new File(destDirname + snmlFilename);
        checkAddressingMode(t, destDir);
        return normalize(exeFile, snmlFile, t, destDir);
    }

    @Override
    public boolean checkResult(SourceFileTest t, File destDir) {
        if (isInvalidProgram(t)) {
            // Check if standard output has an error message.
            String stdout = t.getCompilerStdout();
            boolean okay = stdout != null && stdout.contains("error");
            if (!okay) t.appendFailureMessage("Error message expected");
            return okay;
        }
        boolean okay = true;
        String destDirname = appendDirSep(destDir.getPath());
        for (List<String> list : t.getSourceFileNames()) {
            for (String filename : list) {
                String snmlFilename = normalizedFilename(sFilename(filename));
                String ssolnmlFilename =
                        normalizedFilename(ssolFilename(filename));
                File snmlFile = new File(destDirname + snmlFilename);
                File ssolnmlFile = new File(destDirname + ssolnmlFilename);
                if (!compareFiles(ssolnmlFile, snmlFile, t)) okay = false;
            }
        }
        return okay;
    }

    @Override
    public void printTestResult(SourceFileTest t, File destDir,
            PDFReporter pr) {
        if (isInvalidProgram(t))
            // Nothing to report
            return;
        String destDirname = appendDirSep(destDir.getPath());
        for (List<String> compilationUnit : t.getSourceFileNames())
            for (String filename : compilationUnit) {
                // .s
                String sFilename = sFilename(filename);
                // .s.nml
                String snmlFilename = normalizedFilename(sFilename);
                pr.printHeader("Generated result:");
                pr.printCode(new File(destDirname + snmlFilename));
                // .ssol.nml
                String ssolnmlFilename =
                        normalizedFilename(ssolFilename(filename));
                pr.printHeader("Expected result:");
                pr.printCode(new File(destDirname + ssolnmlFilename));
            }
    }
}
